"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var integratedController_1 = require("../performance/integratedController");
var idleQueue_1 = require("./idleQueue");
var logPerf_1 = require("../helpers/logPerf");
var utils_1 = require("../helpers/utils");
var default_1 = /** @class */ (function () {
    function default_1() {
        if (!utils_1.supportPerformance())
            throw Error("browser doesn't support performance api");
        this.integratedController = new integratedController_1.default();
        this.idleQueue = new idleQueue_1.default();
    }
    default_1.prototype.integratedConfig = function (config) {
        return __awaiter(this, void 0, void 0, function () {
            var projectName, version, firstPaint, firstContentfulPaint, firstInputDelay, timeToInteractive, firstMeaningfulPaint, largestContentfulPaint, timeToFirstByte, navigationTiming, perfTracker, collectMetrics, data;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!utils_1.supportPerformanceObserver())
                            throw Error("browser doesn't support performanceObserver api");
                        projectName = config.projectName, version = config.version, firstPaint = config.firstPaint, firstContentfulPaint = config.firstContentfulPaint, firstInputDelay = config.firstInputDelay, timeToInteractive = config.timeToInteractive, firstMeaningfulPaint = config.firstMeaningfulPaint, largestContentfulPaint = config.largestContentfulPaint, timeToFirstByte = config.timeToFirstByte, navigationTiming = config.navigationTiming, perfTracker = config.perfTracker;
                        this.defaultResult = {
                            projectName: projectName,
                            version: version || '',
                        };
                        collectMetrics = [];
                        if (firstPaint || firstContentfulPaint) {
                            collectMetrics.push(this.integratedController.getFirstPaint());
                        }
                        if (firstInputDelay) {
                            collectMetrics.push(this.integratedController.getFirstInputDelay());
                        }
                        if (firstMeaningfulPaint) {
                            collectMetrics.push(this.integratedController.getFirstMeaningFulPaint());
                        }
                        if (timeToInteractive) {
                            collectMetrics.push(this.integratedController.getTimeToInteractive());
                        }
                        if (largestContentfulPaint) {
                            collectMetrics.push(this.integratedController.getLargestContentFulPaint());
                        }
                        if (timeToFirstByte) {
                            collectMetrics.push(this.integratedController.getTimeToFirstByte());
                        }
                        if (navigationTiming) {
                            collectMetrics.push(this.integratedController.getNavigationTiming());
                        }
                        return [4 /*yield*/, Promise.all(collectMetrics)];
                    case 1:
                        data = _a.sent();
                        if (perfTracker) {
                            this.pushTask(function () {
                                logPerf_1.logMetrics(Object.assign({}, _this.defaultResult, {
                                    data: utils_1.flatObjectInArr(data),
                                }), perfTracker);
                            });
                        }
                        else {
                            return [2 /*return*/, utils_1.flatObjectInArr(data)];
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    default_1.prototype.getFCP = function () {
        return this.transformResult(this.integratedController.getFirstPaint);
    };
    default_1.prototype.getFID = function () {
        return this.transformResult(this.integratedController.getFirstInputDelay);
    };
    default_1.prototype.getFMP = function () {
        return this.transformResult(this.integratedController.getFirstMeaningFulPaint);
    };
    default_1.prototype.getTTI = function () {
        return this.transformResult(this.integratedController.getTimeToInteractive);
    };
    default_1.prototype.getTTFB = function () {
        return this.transformResult(this.integratedController.getTimeToFirstByte);
    };
    default_1.prototype.getLCP = function () {
        return this.transformResult(this.integratedController.getLargestContentFulPaint);
    };
    default_1.prototype.getNavTiming = function () {
        return this.transformResult(this.integratedController.getNavigationTiming);
    };
    default_1.prototype.safteyTracker = function () { };
    default_1.prototype.transformResult = function (fn) {
        return __awaiter(this, void 0, void 0, function () {
            var data;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, fn.call(this.integratedController)];
                    case 1:
                        data = _a.sent();
                        return [2 /*return*/, Object.assign({}, this.defaultResult, data)];
                }
            });
        });
    };
    default_1.prototype.pushTask = function (cb) {
        if (this.idleQueue && this.idleQueue.pushTask) {
            this.idleQueue.pushTask(function () {
                cb();
            });
        }
        else {
            cb();
        }
    };
    return default_1;
}());
exports.default = default_1;
//# sourceMappingURL=Monitor.js.map