"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var idleCallback_1 = require("../helpers/idleCallback");
var utils_1 = require("../helpers/utils");
var DEFAULT_MIN_TASK_TIME = 0;
var IdleQueue = /** @class */ (function () {
    function IdleQueue(_a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.ensureTaskRun, ensureTaskRun = _c === void 0 ? false : _c, _d = _b.defaultTaskTime, defaultTaskTime = _d === void 0 ? DEFAULT_MIN_TASK_TIME : _d;
        this.taskQueue = [];
        this.idleCallbackHandle = null;
        this.processing = false;
        this.defaultMinTaskTime = defaultTaskTime;
        this.ensureTaskRun = ensureTaskRun;
        this.runTask = this.runTask.bind(this);
    }
    IdleQueue.prototype.pushTask = function (cb) {
        this.addTask(Array.prototype.push, cb);
    };
    IdleQueue.prototype.addTask = function (arrayMethod, task, minTaskTime) {
        if (minTaskTime === void 0) { minTaskTime = this.defaultMinTaskTime; }
        var state = {
            time: utils_1.now(),
            visibilityState: document.visibilityState
        };
        arrayMethod.call(this.taskQueue, { task: task, state: state, minTaskTime: minTaskTime });
        this.scheduleTasksToRun();
    };
    IdleQueue.prototype.scheduleTasksToRun = function () {
        if (!this.idleCallbackHandle) {
            this.idleCallbackHandle = idleCallback_1.requestIdleCb(this.runTask);
        }
    };
    IdleQueue.prototype.cancelScheduleRun = function () {
        idleCallback_1.cancelIdleCb(this.idleCallbackHandle);
        this.idleCallbackHandle = null;
    };
    IdleQueue.prototype.runTask = function (deadline) {
        this.cancelScheduleRun();
        if (!this.processing) {
            this.processing = true;
            while (this.hasPendingTasks() && deadline && (deadline.timeRemaining() > this.defaultMinTaskTime || deadline.didTimeout)) {
                var task = this.taskQueue.shift().task;
                task();
            }
            this.processing = false;
            if (this.hasPendingTasks()) {
                this.scheduleTasksToRun();
            }
        }
    };
    IdleQueue.prototype.hasPendingTasks = function () {
        return this.taskQueue.length > 0;
    };
    IdleQueue.prototype.shouldYield = function (deadline, minTaskTime) {
        return deadline && deadline.timeRemaining() <= minTaskTime;
    };
    return IdleQueue;
}());
exports.default = IdleQueue;
//# sourceMappingURL=idleQueue.js.map