"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cancelIdleCb = exports.requestIdleCb = void 0;
var utils_1 = require("./utils");
var IdleDeadline = /** @class */ (function () {
    /** @param {number} initTime */
    function IdleDeadline(initTime) {
        this.initTime = initTime;
    }
    Object.defineProperty(IdleDeadline.prototype, "didTimeout", {
        /** @return {boolean} */
        get: function () {
            return false;
        },
        enumerable: false,
        configurable: true
    });
    /** @return {number} */
    IdleDeadline.prototype.timeRemaining = function () {
        return Math.max(0, 50 - (utils_1.now() - this.initTime));
    };
    return IdleDeadline;
}());
var supportRequestIdleCallback = typeof window.requestIdleCallback === 'function';
/**
 * A minimal shim for the requestIdleCallback function. This accepts a
 * callback function and runs it at the next idle period, passing in an
 * object with a `timeRemaining()` method.
 * @private
 * @param {!Function} callback
 * @return {number}
 */
var requestIdleCallbackShim = function (cb) {
    var deadline = new IdleDeadline(utils_1.now());
    return setTimeout(function () { return cb(deadline); }, 0);
};
/**
 * A minimal shim for the  cancelIdleCallback function. This accepts a
 * handle identifying the idle callback to cancel.
 * @private
 * @param {number|null} handle
 */
var cancelIdleCallbackShim = function (handle) {
    clearTimeout(handle);
};
/**
 * The native `requestIdleCallback()` function or `cancelIdleCallbackShim()`
 *.if the browser doesn't support it.
 * @param {!Function} callback
 * @return {number}
 */
exports.requestIdleCb = supportRequestIdleCallback ? window.requestIdleCallback : requestIdleCallbackShim;
/**
 * The native `cancelIdleCallback()` function or `cancelIdleCallbackShim()`
 * if the browser doesn't support it.
 * @param {number} handle
 */
exports.cancelIdleCb = supportRequestIdleCallback ? window.cancelIdleCallback : cancelIdleCallbackShim;
//# sourceMappingURL=idleCallback.js.map