"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../helpers/utils");
var tagWeightMap = new Map([
    ['VIDEO', 5],
    ['CANVAS', 5],
    ['IMG', 2],
    ['SVG', 2],
]);
var IGNORE_TAG_LIST = ['SCRIPT', 'STYLE', 'META', 'HEAD', 'LINK'];
var FMP_TAG = 'fmp_tag';
var FMP = /** @class */ (function () {
    function FMP() {
        this.markCount = 0;
        this.statusObserve = [];
        this.entries = {};
        this.resolveFn = null;
        // 是否有标记元素
        if (this.checkMarkStatus(document.body)) {
            this.activeMark();
        }
        else {
            this.passiveMark();
        }
    }
    FMP.prototype.getFirstMeaningFulPaint = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.resolveFn = resolve;
            if (document.readyState === 'complete') {
                _this.getCoreElement();
            }
            else {
                window.addEventListener('load', function () {
                    _this.getCoreElement();
                });
            }
        });
    };
    FMP.prototype.checkMarkStatus = function (element) {
        if (element.getAttribute(FMP_TAG))
            return true;
        var children = element.children;
        for (var i = 0; i < children.length; i++) {
            var child = children[i];
            if (this.checkMarkStatus(child))
                return true;
        }
        return false;
    };
    FMP.prototype.getCoreElement = function () {
        var _this = this;
        if (!utils_1.supportMutationObserver())
            throw new Error("browser doesn't support mutationObserver api");
        this.observe.disconnect();
        performance.getEntries().forEach(function (entry) {
            _this.entries[entry.name] = entry.responseEnd;
        });
        var tagEle = this.getTreeWeight(document.body);
        var maxWeightEle = null;
        tagEle.childList.forEach(function (child) {
            if (maxWeightEle && maxWeightEle.weightScore) {
                if (child.weightScore > maxWeightEle.weightScore) {
                    maxWeightEle = child;
                }
            }
            else {
                maxWeightEle = child;
            }
        });
        if (!maxWeightEle) {
            this.resolveFn(0);
            return;
        }
        var els = this.filterEls(maxWeightEle.elementArr);
        this.resolveFn(this.getElementTiming(els));
    };
    FMP.prototype.activeMark = function () { };
    FMP.prototype.passiveMark = function () {
        var _this = this;
        this.getFirstSnapShot();
        this.observe = new MutationObserver(function () {
            var time = performance.now();
            _this.markCount++;
            _this.statusObserve.push({ time: time });
            _this.setTag(document.body, _this.markCount);
        });
        this.observe.observe(document, {
            childList: true,
            subtree: true,
        });
    };
    FMP.prototype.getFirstSnapShot = function () {
        var time = performance.now();
        this.setTag(document.body, this.markCount);
        this.statusObserve.push({ time: time });
    };
    FMP.prototype.getTreeWeight = function (element) {
        if (!element)
            return null;
        var list = [], children = element.children;
        for (var i = 0; i < children.length; i++) {
            var child = children[i];
            if (!child.getAttribute(FMP_TAG))
                continue;
            var elementArr = this.getTreeWeight(child);
            if (elementArr.weightScore)
                list.push(elementArr);
        }
        return this.calculateScore(element, list);
    };
    FMP.prototype.setTag = function (element, count) {
        var tagName = element.tagName;
        if (IGNORE_TAG_LIST.indexOf(tagName) > -1)
            return;
        var children = element.children;
        for (var i = 0; i < children.length; i++) {
            var child = children[i];
            if (!child.getAttribute(FMP_TAG)) {
                if (!utils_1.calculateAreaPrecent(child))
                    continue;
                child.setAttribute(FMP_TAG, "" + count);
            }
            this.setTag(child, count);
        }
    };
    FMP.prototype.getElementTiming = function (els) {
        var _this = this;
        var result = 0;
        els.forEach(function (el) {
            var time = 0;
            if (el.weight === 1) {
                var index = parseInt(el.element.getAttribute(FMP_TAG), 10);
                time = _this.statusObserve[index].time;
            }
            else if (el.weight === 2) {
                if (el.element.tagName === 'IMG') {
                    time = _this.entries[el.element.src];
                }
                else if (el.element.tagName === 'SVG') {
                    var index = parseInt(el.element.getAttribute(FMP_TAG), 10);
                    time = _this.statusObserve[index].time;
                }
            }
            else if (el.weight === 5) {
                if (el.element.tagName === 'VIDEO') {
                    time = _this.entries[el.element.src];
                }
                else if (el.element.tagName === 'CANVAS') {
                    var index = parseInt(el.element.getAttribute(FMP_TAG), 10);
                    time = _this.statusObserve[index].time;
                }
            }
            result = Math.max(result, time);
        });
        return result;
    };
    FMP.prototype.filterEls = function (els) {
        if (els.length === 1)
            return els;
        var sum = 0;
        els.forEach(function (el) {
            sum += el.weightScore;
        });
        var avg = sum / els.length;
        return els.filter(function (el) { return el.weightScore > avg; });
    };
    FMP.prototype.calculateScore = function (element, list) {
        var _a = element.getBoundingClientRect(), width = _a.width, height = _a.height;
        var weight = tagWeightMap.get(element.tagName) || 1, childScore = 0;
        // 子节点总得分
        list.forEach(function (el) {
            childScore += el.weightScore;
        });
        // 节点得分 (宽 * 高 * 权重 * 视图面积比例)
        var weightScore = utils_1.calculateAreaPrecent(element)
            ? width * height * weight * utils_1.calculateAreaPrecent(element)
            : 0;
        var elementArr = [{ element: element, weight: weight, weightScore: weightScore }];
        // 如果子节点总分大于节点得分，核心节点在子节点中
        if (weightScore < childScore || weightScore === 0) {
            weightScore = childScore;
            elementArr = [];
            list.forEach(function (el) {
                elementArr = elementArr.concat(el.elementArr);
            });
        }
        element.setAttribute('fmp_weight', "" + weightScore);
        return {
            weightScore: weightScore,
            elementArr: elementArr,
            childList: list,
            element: element,
        };
    };
    return FMP;
}());
exports.default = (function () { return __awaiter(void 0, void 0, void 0, function () {
    var instance;
    return __generator(this, function (_a) {
        instance = new FMP();
        return [2 /*return*/, Promise.resolve(instance.getFirstMeaningFulPaint())];
    });
}); });
//# sourceMappingURL=fmp.js.map